// image2svgt.cpp : main project file.

using namespace System;
using namespace System::IO;
using namespace System::Text;
using namespace System::Drawing;
using namespace System::Drawing::Imaging;

void convert(String ^filename, String ^out_filename) {
	if( File::Exists(out_filename) ) {
		Console::WriteLine("file " + out_filename + " already exists");
		return;
	}
	FileStream^ file;
	array<Byte>^ data;

	try {
		file = gcnew FileStream(filename, FileMode::Open, FileAccess::Read);
		data = gcnew array<Byte>((int)(file->Length));
		long bytesRead = file->Read(data, 0, (int)file->Length);
		Console::WriteLine("read " + bytesRead + " bytes");
		file->Close();
	}
	catch( Exception^ exp ) {
		Console::WriteLine( " {0}", exp->Message );
		return;
	}

	// we also need to read properties about the Image
	Image ^image;
	try {
		image = Image::FromFile(filename);
	}
	catch( Exception^ exp ) {
		Console::WriteLine( " {0}", exp->Message );
		return;
	}

	bool warn = true;
	String ^format;
	if( image->RawFormat->Equals(ImageFormat::Png) ) {
		format = "png";
		warn = false;
	}
	else if( image->RawFormat->Equals(ImageFormat::Jpeg) ) {
		format = "jpeg";
		warn = false;
	}
	else if( image->RawFormat->Equals(ImageFormat::Bmp) ) {
		format = "bmp";
	}
	else if( image->RawFormat->Equals(ImageFormat::Gif) ) {
		format = "gif";
	}
	else if( image->RawFormat->Equals(ImageFormat::Tiff) ) {
		format = "tiff";
	}
	else {
		Console::WriteLine("unknown file format\n");
		return;
	}
	if( warn ) {
		Console::WriteLine("WARNING - images of type " + format + " are not guaranteed to be supported by SVG Tiny implementations\n");
	}

	FileStream ^outFilestream;
	StreamWriter ^writer;
	try {
		//outFilestream = gcnew FileStream(out_filename, FileMode::CreateNew);
		outFilestream = File::Create(out_filename);
		writer = gcnew StreamWriter(outFilestream);
	}
	catch ( Exception^ exp ) 
	{
		Console::WriteLine( " {0}", exp->Message );
	}

	// Convert the binary input into Base64 UUEncoded output.
	String^ base64String;
	try {
		base64String = Convert::ToBase64String(data, 0, data->Length);
	}
	catch ( ArgumentNullException^ ) {
		Console::WriteLine( "Binary data array is null." );
		return;
	}

	writer->WriteLine("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>");
	writer->WriteLine("<!-- Converted with image2svgt -->");
	writer->WriteLine("<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1 Tiny//EN\" \"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11-tiny.dtd\">");
	writer->WriteLine("<svg");
	writer->WriteLine("    xmlns=\"http://www.w3.org/2000/svg\"");
	writer->WriteLine("    xmlns:xlink=\"http://www.w3.org/1999/xlink\"");
	writer->WriteLine("    version=\"1.1\"");
	writer->WriteLine("    width=\"" + image->Width + "\"");
	writer->WriteLine("    height=\"" + image->Height + "\">");
	writer->Write("<image xlink:href=\"data:image/");
	writer->Write(format);
	writer->Write(";base64,");
	writer->Write(base64String);
	writer->WriteLine("\" width = \"" + image->Width + "\" height = \"" + image->Height + "\" />");
	writer->WriteLine("</svg>");

	writer->Close();
	outFilestream->Close();
	Console::WriteLine("converted successfully!\n");
}

int main(array<System::String ^> ^args) {
//int main(int argc, char *argv[]) {
    //Console::WriteLine(L"Hello World");
	Console::WriteLine("image2svgt version 1.0 by Mark Harman");
	Console::WriteLine("( http://homepage.ntlworld.com/mark.harman/comp_image2svgt.html )");
	int argc = args->Length;
	if( argc < 1 ) {
		Console::WriteLine("Please supply an image file");
		return 0;
	}
	else if( argc < 2 ) {
		Console::WriteLine("Please supply an image file and output file");
		return 0;
	}
	else if( argc > 2 ) {
		Console::WriteLine("Too many arguments");
		return 0;
	}
	else {
		convert(args[0], args[1]);
	}
	return 0;
}
